;;; $Id: exodus.scm,v 3.13 1996/03/31 12:30:58 queinnec Exp $ 
;;; Copyright (c) 1990-96 by Christian Queinnec. All rights reserved.

;;;                ***********************************************
;;;                    Small, Efficient and Innovative Class System
;;;                                    Meroon 
;;;                              Christian Queinnec  
;;;                    Ecole Polytechnique & INRIA--Rocquencourt
;;;                ************************************************

;;; This file (exodus follows genesis) finishes to build the initial
;;; and internal classes of Meroon. Classes can now be defined more
;;; handily with the define-class macro. 

;;;======================================================== fundamental classes
;;; All the fundamental classes were already built by hand but only a
;;; handful of accessors were defined. We can now generate all of them
;;; since all the machinery is present. Don't redefine Object which is 
;;; deeply built-in.

;;; 0
;;;(define-class Object #f ())

;;; 1
(define-class Class Object 
  ((= name              :immutable) ; 0
   (= number            :immutable) ; 1
   (= depth             :immutable) ; 2
   (= fields            :immutable) ; 3
   (= super-number      :immutable) ; 4
   (= subclass-numbers  :mutable)   ; 5
   (= next              :mutable)   ; 6
   (= allocator         :immutable) ; 7
   (= immutable?        :immutable) ; 8
   (* suprel            :immutable) ; 9
   ) )

;;; 2
(define-class Handy-Class Class ())

;;; 3
(define-class MeroonV2-Class Handy-Class ())

;;;4                                                   FUTURE
(define-class Applyable-Object Object
  () )

;;; 5
(define-class Generic Applyable-Object
  ((= name              :immutable)  ;0
   (= default           :mutable)    ;1
   (= variables         :immutable)  ;2
   (= dispatcher        :mutable)    ;3
   (= top-classes       :immutable)  ;4
   ) )

;;; 6
(define-class Generic-1 Generic
  ( ) )

;;; 7
(define-class Generic-N Generic 
  () )

;;; 8
(define-class Field Object 
  ((= immutable?        :immutable)
   (= name              :immutable)
   (= class-number      :mutable)
   (= initialized?      :immutable)
   (= initializer       :mutable)
   (* path              :immutable)
   ) )

;;; 9
(define-class Mono-Field Field ())

;;; 10
(define-class Poly-Field Field ())

;;;=============================================================== Dispatchers
;;; Generic functions contain a dispatcher field, instances of the
;;; following classes:

;;; 11
(define-class Dispatcher Object 
  ((= method-finder     :immutable)
   ) )

;;; 12
(define-class Immediate-Dispatcher Dispatcher 
  ((= method            :immutable)
   ) )

;;; 13
(define-class Subclass-Dispatcher Dispatcher 
  ((= class-number      :immutable) ; 1
   (= class-depth       :immutable) ; 2
   (= no                :mutable)   ; 3
   (= yes               :mutable)   ; 4
   ) )

;;; 14
(define-class Indexed-Dispatcher Dispatcher 
  ((= class-number      :immutable) ; 1
   (= class-depth       :immutable) ; 2
   (= no                :mutable)   ; 3
   (* method            :mutable)   ; 4
   ) )

;;; 15
(define-class Linear-Dispatcher Dispatcher
  ((= no                :mutable)
   (= method            :mutable)
   (= signature         :mutable)
   ) )

;;; 16
(define-class Global-Dispatcher Dispatcher
  ( (* method           :mutable)
    ) )

;;;17
(define-class Tracing-Dispatcher Dispatcher
  ( (= dispatcher       :mutable)
    (= default          :immutable)
    ) )

;;;=============================================================== Anomaly
;;; Finally all errors are handled through this class.

;;; 18
(define-class Anomaly Object 
  ( (= category         :immutable)
    (= operator         :immutable)
    (= message          :immutable)
    (* hint             :immutable)
    ) )

;;;19
(define-class Warning Anomaly
  ( ) )

;;; end of exodus.scm
