;;; $Id: instance.gsc,v 3.1 1996/05/18 15:52:23 queinnec Exp $
;;; Copyright (c) 1990-96 by Christian Queinnec. All rights reserved.

;;;                ***********************************************
;;;                    Small, Efficient and Innovative Class System
;;;                                    Meroon V3
;;;                              Christian Queinnec  
;;;                   \'Ecole Polytechnique & INRIA--Rocquencourt
;;;                ************************************************

;;;                 Gambit-C adaptation of instance.scm

;;; Use a special subtype for Meroon objects. This allows the Object?
;;; predicate to be safe (so the safer-object feature is just a waste
;;; of room that should be avoided). Thanks to Brad Lucier
;;; <lucier@MATH.Purdue.EDU> for these files.

;;; This file only contains functions. Associated macros are in the 
;;; macinst.gsc file.

;;; Test if a Scheme object is a Meroon object. 

(define (Object? o)
  (and (##subtyped? o)
       (##eq? (##subtype o) (subtype-meroon)) ) )

;;; Extracts the internal number of a class from within an instance.

(define (object->class-number o)
  (##vector-ref o 0) )

;;; Shallow copy of an instance.

(define (instance-clone o)
  (let* ((n (##vector-length o))
         (r (##make-vector n meroon-uninitialized)) )
    (##subtype-set! r (subtype-meroon))
    (copy-instance-slice! o r 0 n)
    r ) )

;;; Copy instance old[start..end[ into instance new[start..end[

(define (copy-instance-slice! old new start end)
  (let copy ((i start))
    (when (##fixnum.< i end)
      (##vector-set! new i (##vector-ref old i))
      (copy (##fixnum.+ i 1)) ) ) )

;;; Local Variables:
;;; mode: scheme
;;; End:

;;; end of instance.gsc
