;;; $Id: macinst.gsc,v 3.4 1997/12/29 09:31:11 queinnec Exp $
;;; Copyright (c) 1990-96 by Christian Queinnec. All rights reserved.

;;;                ***********************************************
;;;                    Small, Efficient and Innovative Class System
;;;                                    Meroon V3
;;;                              Christian Queinnec  
;;;                   \'Ecole Polytechnique & INRIA--Rocquencourt
;;;                ************************************************

;;;                 Gambit-C adaptation of macinst.scm

;;; Use a special subtype for Meroon objects. This allows the Object?
;;; predicate to be safe (so the safer-object feature is just a waste
;;; of room that should be avoided). Thanks to Brad Lucier
;;; <lucier@MATH.Purdue.EDU> for these files.

(if-meroon-feature (and gambit C (or 2.7))
  (define-meroon-macro (subtype-meroon) 6)
  (define-meroon-macro (subtype-meroon) 12) )

(if-meroon-feature safer-object
  (/ "The safer-object feature must not be used with Gambit-C" 0)
  #f )

;;; This variable designatest the first offset in a vector to hold a slot.
;;; It usually is 1 (since 0 is reserved to the instantiation link).

(define-meroon-macro (starting-offset)
  1 )

;;; Test if a Scheme object is a Meroon object. 
;;; Access to instances. Since all accesses are checked, these
;;; functions would better avoid to check that o is a struct, that the
;;; offset is a natural and that the offset lies within the struct.

(define-meroon-macro (instance-ref o offset)
  (if (integer? offset)
      `(##vector-ref ,o ,(+ (starting-offset) offset))
      `(##vector-ref ,o (##fixnum.+ (starting-offset) ,offset))) )

(define-meroon-macro (instance-set! o offset value)
  (if (integer? offset)
      `(##vector-set! ,o ,(+ (starting-offset) offset) ,value)
      `(##vector-set! ,o (##fixnum.+ (starting-offset) ,offset) ,value)) )

;;; Builds an instance with CN as instantiation link and ARGS as content.
;;; ARGS are values for mono- or poly- fields as well as their size.
;;; Since vectors are initialized with the undefined value, it is
;;; needless to reinitialize them with this undefined value.

(define-meroon-macro (instance cn . args)
  `(##subtype-set! (##vector ,cn ,@args) (subtype-meroon)) )

;;; Allocate an empty instance with CN as instantation link. Caution,
;;; the result is not necessarily a well formed instance. It might
;;; need to be skeletized if containing poly-fields.

(define-internal-meroon-macro (allocate-empty-instance cn size)
  (let ((g (gensym)))
    `(let ((,g (##make-vector (##fixnum.+ (starting-offset) ,size) 
                              meroon-uninitialized )))
       (##subtype-set! ,g (subtype-meroon))
       (##vector-set! ,g 0 ,cn)
       ,g ) ) )

;;; Allocate an instance with a given CONTENT. Similar to instance,
;;; except that CONTENT is given as a list. 

(define-internal-meroon-macro (allocate-full-instance cn content)
  (let ((g (gensym))
        (c (gensym)) )
    `(let* ((,c ,content)
            (,g (allocate-empty-instance ,cn (##length ,c))) )
       (fill-instance! ,g 0 ,c) ) ) )

;;; This one is actually needed but curious. It is actually needed by
;;; maker.scm to check a posteriori the allocation of an instance. It
;;; is also used by size.scm to know the size of Meroon. It returns
;;; the number of values contained in an instance (instantiation link
;;; not comprised).

(define-internal-meroon-macro (instance-length o)
  `(##fixnum.- (##vector-length ,o) (starting-offset)) )

;;; Local Variables:
;;; mode: scheme
;;; End:

;;; end of macinst.gsc
